@@ -1,21 +1,22 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/mp.c,v 1.2 2002/06/14 21:02:41 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/mp.c,v 1.4 2003/01/08 20:40:54 richb Exp $
  *
- *  Copyright (c) 1987-2002, Sun Microsystems, Inc.  All Rights Reserved.
- *  Sun considers its source code as an unpublished, proprietary
- *  trade secret, and it is available only under strict license
- *  provisions.  This copyright notice is placed here only to protect
- *  Sun in the event the source is deemed a published work.  Dissassembly,
- *  decompilation, or other means of reducing the object code to human
- *  readable form is prohibited by the license agreement under which
- *  this code is provided to the user or company in possession of this
- *  copy.
- *
- *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the
- *  Government is subject to restrictions as set forth in subparagraph
- *  (c)(1)(ii) of the Rights in Technical Data and Computer Software
- *  clause at DFARS 52.227-7013 and in similar clauses in the FAR and
- *  NASA FAR Supplement.
+ *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
+ *           
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *           
+ *  This program is distributed in the hope that it will be useful, but 
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ *  General Public License for more details.
+ *           
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  02111-1307, USA.
  */
 
 /*  This maths library is based on the MP multi-precision floating-point
@@ -1171,9 +1172,14 @@ L30:
 void
 mpcmim(int *x, int *y)
 {
+    int tmp[MP_SIZE];     /* Temporary store for the number. */
+    int accuracy;         /* Temporary story for the accuracy. */
+    char disp[MAXLINE];   /* Setup a string to store what would be displayed */
+    enum num_type dtype;  /* Setup a temp display type variable */
+
     int i__1;
 
-    static int i, il;
+    static int i, il, ll;
 
 /* RETURNS Y = INTEGER PART OF X (TRUNCATED TOWARDS 0), FOR MP X AND Y.
  * USE IF Y TOO LARGE TO BE REPRESENTABLE AS A SINGLE-PRECISION INTEGER. 
@@ -1186,17 +1192,24 @@ mpcmim(int *x, int *y)
 
     mpchk(&c__1, &c__4);
     mpstr(&x[1], &y[1]);
-    if (y[1] == 0) return;
+    if (y[1] == 0) {
+        return;
+    }
 
     il = y[2] + 1;
+    ll = il;
 
 /* IF EXPONENT LARGE ENOUGH RETURN Y = X */
 
-    if (il > MP.t) return;
+    if (il > MP.t) {
+        return;
+    }
 
 /* IF EXPONENT SMALL ENOUGH RETURN ZERO */
 
-    if (il > 1) goto L10;
+    if (il > 1) {
+        goto L10;
+    }
 
     y[1] = 0;
     return;
@@ -1205,7 +1218,27 @@ mpcmim(int *x, int *y)
 
 L10:
     i__1 = MP.t;
-    for (i = il; i <= i__1; ++i) y[i + 2] = 0;
+    for (i = il; i <= i__1; ++i) {
+        y[i + 2] = 0;
+    }
+
+/* Fix for Sun bugtraq bug #4006391 - problem with Int function for numbers
+ * like 4800 when internal representation in something like 4799.999999999.
+ */
+
+    dtype = v->dtype;
+    v->dtype = FIX;
+    accuracy = v->accuracy;
+    v->accuracy = MAX_DIGITS;
+    mpcmf(&x[1], tmp);
+    STRCPY(disp, make_number(tmp, TRUE));
+
+    if (disp[0] == '1') {
+        y[ll+1]++;
+    }
+
+    v->accuracy = accuracy;
+    v->dtype = dtype;
 }
 
 
